@@ -33,7 +33,7 @@ use snafu::{ensure, OptionExt, ResultExt, Snafu};
 
 /// A client that can perform a get request
 #[async_trait]
-pub trait GetClient: Send + Sync + 'static {
+pub(crate) trait GetClient: Send + Sync + 'static {
     const STORE: &'static str;
 
     /// Configure the [`HeaderConfig`] for this client
@@ -44,7 +44,7 @@ pub trait GetClient: Send + Sync + 'static {
 
 /// Extension trait for [`GetClient`] that adds common retrieval functionality
 #[async_trait]
-pub trait GetClientExt {
+pub(crate) trait GetClientExt {
     async fn get_opts(&self, location: &Path, options: GetOptions) -> Result<GetResult>;
 }
 
@@ -96,14 +96,13 @@ impl ContentRange {
 
 /// A specialized `Error` for get-related errors
 #[derive(Debug, Snafu)]
-#[allow(missing_docs)]
 enum GetResultError {
     #[snafu(context(false))]
     Header {
         source: crate::client::header::Error,
     },
 
-    #[snafu(context(false))]
+    #[snafu(transparent)]
     InvalidRangeRequest {
         source: crate::util::InvalidGetRange,
     },
@@ -386,7 +385,7 @@ mod tests {
         let err = get_result::<TestClient>(&path, Some(get_range.clone()), resp).unwrap_err();
         assert_eq!(
             err.to_string(),
-            "InvalidRangeRequest: Wanted range starting at 2, but object was only 2 bytes long"
+            "Wanted range starting at 2, but object was only 2 bytes long"
         );
 
         let resp = make_response(
