@@ -85,7 +85,7 @@ impl From<Error> for crate::Error {
 
 /// Internal client for HttpStore
 #[derive(Debug)]
-pub struct Client {
+pub(crate) struct Client {
     url: Url,
     client: reqwest::Client,
     retry_config: RetryConfig,
@@ -93,7 +93,11 @@ pub struct Client {
 }
 
 impl Client {
-    pub fn new(url: Url, client_options: ClientOptions, retry_config: RetryConfig) -> Result<Self> {
+    pub(crate) fn new(
+        url: Url,
+        client_options: ClientOptions,
+        retry_config: RetryConfig,
+    ) -> Result<Self> {
         let client = client_options.client()?;
         Ok(Self {
             url,
@@ -103,7 +107,7 @@ impl Client {
         })
     }
 
-    pub fn base_url(&self) -> &Url {
+    pub(crate) fn base_url(&self) -> &Url {
         &self.url
     }
 
@@ -159,7 +163,7 @@ impl Client {
         Ok(())
     }
 
-    pub async fn put(
+    pub(crate) async fn put(
         &self,
         location: &Path,
         payload: PutPayload,
@@ -216,7 +220,7 @@ impl Client {
         }
     }
 
-    pub async fn list(&self, location: Option<&Path>, depth: &str) -> Result<MultiStatus> {
+    pub(crate) async fn list(&self, location: Option<&Path>, depth: &str) -> Result<MultiStatus> {
         let url = location
             .map(|path| self.path_url(path))
             .unwrap_or_else(|| self.url.clone());
@@ -255,7 +259,7 @@ impl Client {
         Ok(status)
     }
 
-    pub async fn delete(&self, path: &Path) -> Result<()> {
+    pub(crate) async fn delete(&self, path: &Path) -> Result<()> {
         let url = self.path_url(path);
         self.client
             .delete(url)
@@ -271,7 +275,7 @@ impl Client {
         Ok(())
     }
 
-    pub async fn copy(&self, from: &Path, to: &Path, overwrite: bool) -> Result<()> {
+    pub(crate) async fn copy(&self, from: &Path, to: &Path, overwrite: bool) -> Result<()> {
         let mut retry = false;
         loop {
             let method = Method::from_bytes(b"COPY").unwrap();
@@ -364,12 +368,12 @@ impl GetClient for Client {
 
 /// The response returned by a PROPFIND request, i.e. list
 #[derive(Deserialize, Default)]
-pub struct MultiStatus {
+pub(crate) struct MultiStatus {
     pub response: Vec<MultiStatusResponse>,
 }
 
 #[derive(Deserialize)]
-pub struct MultiStatusResponse {
+pub(crate) struct MultiStatusResponse {
     href: String,
     #[serde(rename = "propstat")]
     prop_stat: PropStat,
@@ -377,7 +381,7 @@ pub struct MultiStatusResponse {
 
 impl MultiStatusResponse {
     /// Returns an error if this response is not OK
-    pub fn check_ok(&self) -> Result<()> {
+    pub(crate) fn check_ok(&self) -> Result<()> {
         match self.prop_stat.status.contains("200 OK") {
             true => Ok(()),
             false => Err(Error::PropStatus {
@@ -389,7 +393,7 @@ impl MultiStatusResponse {
     }
 
     /// Returns the resolved path of this element relative to `base_url`
-    pub fn path(&self, base_url: &Url) -> Result<Path> {
+    pub(crate) fn path(&self, base_url: &Url) -> Result<Path> {
         let url = Url::options()
             .base_url(Some(base_url))
             .parse(&self.href)
@@ -413,7 +417,7 @@ impl MultiStatusResponse {
     }
 
     /// Returns this objects metadata as [`ObjectMeta`]
-    pub fn object_meta(&self, base_url: &Url) -> Result<ObjectMeta> {
+    pub(crate) fn object_meta(&self, base_url: &Url) -> Result<ObjectMeta> {
         let last_modified = self.prop_stat.prop.last_modified;
         Ok(ObjectMeta {
             location: self.path(base_url)?,
@@ -425,19 +429,19 @@ impl MultiStatusResponse {
     }
 
     /// Returns true if this is a directory / collection
-    pub fn is_dir(&self) -> bool {
+    pub(crate) fn is_dir(&self) -> bool {
         self.prop_stat.prop.resource_type.collection.is_some()
     }
 }
 
 #[derive(Deserialize)]
-pub struct PropStat {
+pub(crate) struct PropStat {
     prop: Prop,
     status: String,
 }
 
 #[derive(Deserialize)]
-pub struct Prop {
+pub(crate) struct Prop {
     #[serde(deserialize_with = "deserialize_rfc1123", rename = "getlastmodified")]
     last_modified: DateTime<Utc>,
 
@@ -452,6 +456,6 @@ pub struct Prop {
 }
 
 #[derive(Deserialize)]
-pub struct ResourceType {
+pub(crate) struct ResourceType {
     collection: Option<()>,
 }
